@@ -1,10 +1,14 @@
 use musli::{Decode, Encode};
 use musli_json::JsonEncoding;
 
-// Mode marker indicating that some attributes should only apply when we're
+// M marker indicating that some attributes should only apply when we're
 // decoding in a JSON mode.
 mod my_modes {
+    use musli::mode::Mode;
+
     pub(crate) enum Json {}
+
+    impl Mode for Json {}
 }
 
 const CONFIG: JsonEncoding<my_modes::Json> = JsonEncoding::new();
